MacroSequence = MacroSequence or {}

function MacroSequence:Initialize()
  if not self.sequences then
    print("MacroSequences.sequences does not exist. Please make sure there are no errors in Interface\\AddOns\\MacroSequence\\Sequences.lua.")
    return
  end

  for name, data in pairs(self.sequences) do
    self:CreateSequence(name, data)
  end
end

function MacroSequence:CreateSequence(name, data)
  if #data == 0 then
    print(format("MacroSequence %s contains no macros. See Interface\\AddOns\\MacroSequence\\Sequences.lua.", name))
    return
  end
  
  -- Create the button to be /clicked
  local button = CreateFrame(
    "Button",
    name,
    header,
    "SecureActionButtonTemplate"
  )
  button:SetAttribute("type", "macro")
  
  -- Create the secure handler header for the sequence
  local header = CreateFrame("Frame", nil, nil, "SecureHandlerBaseTemplate")
  
  -- Copy the sequence information into the secure environment
  header:Execute([[
    step = 1
    macros = newtable([=======[]]..strjoin("]=======],[=======[", unpack(data))..[[]=======])
  ]])
  
  -- Set up reset conditions for the sequence
  local reset = self:ApplyResetConditions(header, data) or ""
  
  -- Set up overall cycling behavior
  header:WrapScript(button, "OnClick",
    reset..[[
      self:SetAttribute("macrotext", macros[step])
      step = step + 1
      if step > #macros then
        step = cycleTarget or 1
      end
    ]]
  )
end

function MacroSequence:ApplyResetConditions(header, data)
  if #data == 1 then
    -- Reset is meaningless if there's only one entry
    return
  end
  
  -- Make the sequence reset after death
  header:SetScript("OnEvent", function(self)
    self:Execute("step = 1")
  end)
  header:RegisterEvent("PLAYER_DEAD")

  if not data.reset then
    return
  end
  
  self:ApplyCombatReset(header, data)
  self:ApplyCycleReset(header, data)
  
  return self:GetModifierResets(data)..self:GetTimerReset(header, data)
end

function MacroSequence:ApplyCombatReset(header, data)
  if data.reset.combat then
    -- Add event for leaving combat to reset the sequence
    header:RegisterEvent("PLAYER_REGEN_ENABLED")
  end
end

function MacroSequence:ApplyCycleReset(header, data)
  local cycle = tonumber(data.reset.cycle)
  if cycle and cycle < #data then
    header:Execute("cycleTarget = "..(#data - cycle + 1))
  end
end

local modifierAPIs = {
  alt = "IsAltKeyDown()",
  ctrl = "IsControlKeyDown()",
  shift = "IsShiftKeyDown()"
}
function MacroSequence:GetModifierResets(data)
  local checks = ""
  for modifier, api in pairs(modifierAPIs) do
    if data.reset[modifier] then
      checks = checks..[[
        if ]]..api..[[ then
          step = 1
        end
      ]]
    end
  end
  return checks
end

function MacroSequence:GetTimerReset(header, data)
  local seconds = data.reset.seconds
  if seconds and seconds > 0 then
    header:Execute("lastTime = 0")
    return [[
      local time = control:GetTime()
      if time - lastTime > ]]..seconds..[[ then
        step = 1
      end
      lastTime = time
    ]]
  else
    return ""
  end
end

MacroSequence:Initialize()
